﻿
open System
open System.IO
open System.Diagnostics
open Text.RegularExpressions

let numTrials = 10
do printfn "Running %d trials..." numTrials

// Get test inputs
let inputFiles = Directory.GetFiles ("input","*.imj")

// Specify how to start coneqct on any given input file
let procSpec (inp: string) =
  let psi = new ProcessStartInfo() 
  do psi.FileName               <- "coneqct.exe"
  do psi.Arguments              <- inp
  do psi.UseShellExecute        <- false
  do psi.RedirectStandardOutput <- true
  do psi.CreateNoWindow         <- true
  psi

// Specify how to get the right part of the output
let getTime (output: string) : int =
  let m = Regex.Match (output, "Result \w \((\d+) ms\)")
  Int32.Parse (m.Groups.[0].Value)

// Run coneqct on an input converting the output to an int
let run (inp: string) : int =
  let coneqct = Process.Start (procSpec inp)
  do coneqct.WaitForExit ()
  let output = coneqct.StandardOutput.ReadToEnd ()
  getTime output

// Run something n times and accumulate the result
let runManyTimes (n: int) (inp: string) : unit =
  
  // Capture the times
  let times = Array.create n 0
  for i in [0..n-1] do
    times.[i] <- run inp 

  // Calculate the mean
  let mean = (Array.sum times) / n

  // Calculate the variance
  let square n = n * n
  let diffs = Array.map (fun t -> square (t - mean)) times
  let variance = (Array.sum diffs) / n

  // Calculate the standard deviation
  let sd = Math.Sqrt ((float) variance)

  // Print the mean and standard deviation
  printfn "%s: mean = %d, sd = %f" inp mean sd

// Initialize accumulated timers (ms)
// a map from file names to accumlated times 
for inp in inputFiles do
  runManyTimes numTrials inp

